#!/usr/bin/python3
# -*- coding: utf-8 -*-

import pwd
import re

from base.scope import Scope
from base.util.util import Util
from helper import system as sysx

class SshKey:
    def __init__(self):
        scope = Scope().get_instance()
        self.logger = scope.get_logger()
        self.util = Util()
        self.db_service = scope.get_db_service()

    def set_ssh_settings(self, data):
        if data["useSsh"]:
            if not self.util.is_installed("openssh-server"):
                result_code, p_out, p_err = self.util.execute("apt install openssh-server -y")
                if result_code == 0:
                    self.logger.info("useSsh is True and Installed openssh-server successfully")
                else:
                    self.logger.error("An error occurred while installing the openssh-server")
            if "publicKey" in data and data["publicKey"] != None:
                self.set_global_ssh_settings()
            if "sshUsername" in data and data["sshUsername"] != None:
                self.set_authorized_key(data["sshUsername"], data["publicKey"])
            self.update_database(data)
            result_code, p_out, p_err = self.util.execute("systemctl restart ssh.service")
        else:
            self.delete_ssh_settings()

    def set_global_ssh_settings(self):
        sshd_file_config = "/etc/ssh/sshd_config"
        pattern = re.compile(r'\s+')
        if self.util.is_exist(sshd_file_config):
            sshd_file = open(sshd_file_config, 'r')
            file_data = sshd_file.read()
            is_password_authentication_yes = False
            is_password_authentication_no = False
            is_authorized_file = False
            is_authorized_file_line = None
            read_sshd_file = self.util.read_file_by_line(sshd_file_config)
            for line in read_sshd_file:
                text = pattern.sub('', line)
                if "PasswordAuthenticationyes" in text and not line.startswith("#"):
                    is_password_authentication_yes = True
                if "PasswordAuthenticationno" in text:
                    is_password_authentication_no = False
                    if line.startswith("#"):
                        is_password_authentication_no = True
                else:
                    is_password_authentication_no = True
                if "AuthorizedKeysFile" in text:
                    is_authorized_file_line = line.strip()

            if is_authorized_file_line:
                if is_authorized_file_line.startswith("#"):
                    is_authorized_file = True
                    replaced_file = is_authorized_file_line.replace("#", "")
                    is_authorized_file_line = replaced_file + " .ssh/liderahenk_keys"
                else:
                    text = pattern.sub('', is_authorized_file_line)
                    is_authorized_file = True
                    if not ".ssh/liderahenk_keys" in text:
                        is_authorized_file_line = is_authorized_file_line + " .ssh/liderahenk_keys"
                        file_data2 = file_data.replace("AuthorizedKeysFile", "#AuthorizedKeysFile")
                        sshd_file.close()
                        sshd_file = open(sshd_file_config, 'w')
                        sshd_file.write(file_data2)
                        sshd_file.close()
                    else:
                        is_authorized_file = False
            else:
                is_authorized_file = True
                is_authorized_file_line = "AuthorizedKeysFile	.ssh/authorized_keys .ssh/authorized_keys2 .ssh/liderahenk_keys"

            if is_authorized_file:
                self.util.write_file(sshd_file_config, "\n" + is_authorized_file_line, mode='a')

            if is_password_authentication_yes:
                file_data = file_data.replace("PasswordAuthentication", "#PasswordAuthentication")
                sshd_file.close()
                sshd_file = open(sshd_file_config, 'w')
                sshd_file.write(file_data)
                sshd_file.close()
                is_password_authentication_no = True
            if is_password_authentication_no:
                if not "PasswordAuthentication no" in read_sshd_file:
                    self.util.write_file(sshd_file_config, "\nPasswordAuthentication no", mode='a')
                    self.logger.info("Configured sshd config file as PasswordAuthentication no")
        else:
            self.logger.info("{0} file not found".format(sshd_file_config))

    def set_authorized_key(self, sshUsername, publicKey):
        for p in pwd.getpwall():
            if not sysx.shell_is_interactive(p.pw_shell):
                continue
            if p.pw_uid == 0:
                continue
            if sshUsername == p.pw_name:
                home_path = self.util.get_homedir(sshUsername)
                self.logger.info("Get home directory: " + str(home_path))
                if not self.util.is_exist(home_path + "/.ssh"):
                    self.util.create_directory(home_path + "/.ssh")
                    self.util.change_owner(home_path + "/.ssh", sshUsername)
                if not self.util.is_exist(home_path + "/.ssh/liderahenk_keys"):
                    self.util.create_file(home_path + "/.ssh/liderahenk_keys")
                    self.util.change_owner(home_path + "/.ssh/liderahenk_keys", sshUsername)
                    self.logger.info("Created liderahenk_keys file and changed permission")
                self.util.write_file(home_path + "/.ssh/liderahenk_keys", publicKey, mode='w')
                self.logger.info("Added public key to liderahenk_keys file")
                break
            else:
                self.logger.info("Username not found: {0}".format(sshUsername))

    def update_database(self, data):
        try:
            rows = self.db_service.select('ssh_key', '*')
            ssh_key_columns = self.db_service.get_cols('ssh_key')
            if rows:
                self.logger.info("key columns: " + str(ssh_key_columns))
                isExist = False
                for key in rows:
                    if key[1] == data["keyVersion"]:
                        isExist = True
                        break
                if not isExist:
                    self.db_service.delete('ssh_key', None)
                    self.db_service.update('ssh_key', ssh_key_columns,
                                           [data["keyVersion"], data["publicKey"], data["sshUsername"], data["useSsh"]])
                    self.logger.info("Updated public key in db successfully")
                else:
                    self.logger.info("Key already exist")
            else:
                self.db_service.update('ssh_key', ssh_key_columns,
                                       [data["keyVersion"], data["publicKey"], data["sshUsername"], data["useSsh"]])
                self.logger.info("Added key to db successfully")
        except Exception as e:
            self.logger.error("An error occurred while updating the database for key. Error: " + str(e))

    def delete_ssh_settings(self):
        self.sshUsername = None
        rows = self.db_service.select('ssh_key', '*')
        if rows:
            for key in rows:
                self.sshUsername = key[3]
            self.db_service.delete('ssh_key', None)
        if self.util.is_installed("openssh-server"):
            self.util.execute("apt purge openssh-server -y")
            self.logger.info("Uninstalled openssh-server package successfully")
        if self.sshUsername:
            home_dir = self.util.get_homedir(self.sshUsername)
            if home_dir:
                if self.util.is_exist(home_dir + "/.ssh/liderahenk_keys"):
                    self.util.delete_file(home_dir + "/.ssh/liderahenk_keys")
                    self.logger.info("Deleted authorized_key successfully")
