#!/usr/bin/python3
# -*- coding: utf-8 -*-

from base.scope import Scope
from base.util.util import Util

import datetime
import subprocess

class SslCertManagement:
    def __init__(self):
        scope = Scope().get_instance()
        self.logger = scope.get_logger()
        self.util = Util()
        self.configuration_manager = scope.get_configuration_manager()
        self.tls_trust_certs_file_path = str(self.configuration_manager.get('PULSAR', 'tls_trust_certs_file_path'))
        self.host = "eba.gov.tr"
        self.days = 7
        self.port = 443

    def get_certificate_expiry_date(self):
        """
        Get the expiry date of the SSL certificate and check if it is about to expire.
        If the certificate is about to expire or does not exist, download a new one.
        """
        try:
            if self.configuration_manager.get('PULSAR', 'pulsar_use_tls').strip().lower() == 'false':
                return
            
            if not self.util.is_exist(self.tls_trust_certs_file_path):
                self.logger.error(f"Certificate file does not exist: {self.tls_trust_certs_file_path}")
                if self.configuration_manager.get('PULSAR', 'pulsar_use_tls').strip().lower() == 'true':
                    self.download_certificate_from_website()
                return

            command = f"openssl x509 -enddate -noout -in {self.tls_trust_certs_file_path}"
            result = subprocess.run(command, shell=True, capture_output=True, text=True)

            if result.returncode != 0:
                self.logger.error(f"Error occurred while getting certificate expiry date: {result.stderr}")
                return

            expiry_str = result.stdout.strip().split("=")[1]
            expiry_date = datetime.datetime.strptime(expiry_str, "%b %d %H:%M:%S %Y %Z")
            remaining_days = (expiry_date - datetime.datetime.now()).days

            if remaining_days <= self.days:
                self.logger.warning(f"WARNING: Certificate expires in {remaining_days} days!")
                if remaining_days < self.days:
                    self.download_certificate_from_website()
            else:
                self.logger.info(f"Certificate is valid for {remaining_days} more days.")

        except Exception as e:
            self.logger.error(f"Failed to control certificate: {str(e)}")
            self.download_certificate_from_website()

    def download_certificate_from_website(self):
        """
        Download the SSL certificate from the specified host and save it to the configured path.
        """
        try:
            command = (
                f"openssl s_client -showcerts -connect {self.host}:{self.port} -servername {self.host} </dev/null 2>/dev/null "
                f"| sed -n '/-.BEGIN/,/-.END/p' > {self.tls_trust_certs_file_path}"
            )
            result = subprocess.run(command, shell=True, capture_output=True, text=True)

            if result.returncode != 0:
                self.logger.error(f"Error occurred while downloading certificate: {result.stderr}")
            else:
                self.logger.info(f"Certificate downloaded from {self.host} and saved to {self.tls_trust_certs_file_path}")

        except Exception as e:
            self.logger.error(f"Error occurred while downloading certificate: {str(e)}")
