#!/usr/bin/python3
# -*- coding: utf-8 -*-
# Author: Volkan Şahin <volkansah.in> <bm.volkansahin@gmail.com>
# Author: Tuncay ÇOLAK <tuncay.colak@tubitak.gov.tr>

import datetime
import json
import re
import uuid
from uuid import getnode as get_mac

from base.scope import Scope
from base.messaging.anonymous_messenger import AnonymousMessenger
from base.system.system import System
from base.timer.setup_timer import SetupTimer
from base.timer.timer import Timer
from base.util.util import Util
from helper import system as sysx
from base.key.ssh_key_management import SshKey
from base.system.disk_info import DiskInfo
import requests
from requests.exceptions import HTTPError, Timeout, ConnectionError
import configparser



class Registration:
    def __init__(self):
        scope = Scope().get_instance()
        self.logger = scope.get_logger()
        self.message_manager = scope.get_message_manager()
        self.event_manager = scope.get_event_manager()
        self.messenger = scope.get_messenger()
        self.conf_manager = scope.get_configuration_manager()
        self.db_service = scope.get_db_service()
        self.register_success_message = False
        self.register_error_message = False
        self.util = Util()
        self.ssh_key = SshKey()
        
        self.event_manager.register_event('REGISTRATION_RESPONSE', self.registration_process)

        if self.is_registered():
            self.logger.debug('Ahenk already registered')
        else:
            self.register(True)

    def registration_request_for_xmpp(self):
        self.logger.debug('Requesting registration')
        SetupTimer.start(Timer(System.Ahenk.registration_timeout(), timeout_function=self.registration_timeout,
                               checker_func=self.is_registered, kwargs=None))
        anon_messenger = AnonymousMessenger(self.message_manager.registration_msg())
        anon_messenger.connect_to_server()

    # registration request with http
    def registration_request(self):
        try:
            messenger_type = self.conf_manager.get('MESSENGER', 'messenger_type', fallback='xmpp').lower()

            if messenger_type == "pulsar":
                self.logger.debug("Pulsar client selected for registration")
                self.registration_request_http()

            else:
                self.logger.debug("XMPP client selected for registration (default)")
                self.registration_request_for_xmpp()

        except Exception as err:
            self.logger.error(f"Error in registration_request: {err}")
            self.stop_service()
                
            
    def registration_request_http(self):
        if self.conf_manager.has_section('REGISTRATION'):
            base_url = str(self.conf_manager.get('REGISTRATION', 'registration_url'))
            ssl_verify = self.conf_manager.getboolean('REGISTRATION', 'ssl_verify', fallback=True)
            ssl_cert_path = str(self.conf_manager.get('REGISTRATION', 'register_tls_trust_certs_file_path', fallback=None))
            
            timeout = self.conf_manager.get('REGISTRATION', 'timeout', fallback=10)
            try:
                timeout = int(timeout)
            except ValueError:
                self.logger.error("Invalid timeout value in configuration. Must be an integer.")
            
            if ssl_verify:
                http_url = "https://{base_url}/register".format(base_url=base_url)
            else:
                http_url = "http://{base_url}/register".format(base_url=base_url)

            payload = self.message_manager.registration_msg()
            self.logger.info(f"Request payload: {payload}")
            headers = {'Content-Type': 'application/json'}
            try:
                if ssl_verify:
                    if ssl_cert_path:
                        response = requests.post(http_url, json=payload, headers=headers, verify=ssl_cert_path, timeout=timeout)
                    else:
                        response = requests.post(http_url, json=payload, headers=headers, verify=True, timeout=timeout)
                else:
                    response = requests.post(http_url, json=payload, headers=headers, verify=False, timeout=timeout)
                response.raise_for_status()
                self.logger.info(f"Response: {response.text}")
                self.registration_process(response.text)

            except requests.exceptions.HTTPError as http_err:
                self.logger.error(f'HTTP error occurred: {http_err}, Status Code: {response.status_code if response else "N/A"}')
            except requests.exceptions.ConnectionError as conn_err:
                self.logger.error(f'HTTP connection error occurred: {conn_err}')
            except requests.exceptions.Timeout:
                self.logger.error('The request timed out')
            except Exception as err:
                self.logger.error(f'Other error occurred: {err}')
        else:
            self.logger.debug("No registration conf under config.d")
            self.stop_service()


    def stop_service(self):
        self.logger.info("Stopping the service...")
        Util.execute("/etc/init.d/ahenk stop")
        
    def ldap_registration_request(self):
        self.logger.debug('Requesting LDAP registration')
        self.messenger.send_Direct_message(self.message_manager.ldap_registration_msg())

    def registration_process(self, reg_reply):
        self.logger.debug('Reading registration reply')
        j = json.loads(reg_reply)
        self.logger.debug('[Registration]' + j['message'])
        status = str(j['status']).lower()
        dn = str(j['agentDn'])
        self.logger.debug('Registration status: ' + str(status))
        # user = Util.get_active_user()
        # display_number = Util.get_username_display_gnome(user)
        # self.logger.info("Display number is {0} number for {1}".format(display_number, user))
        re.compile(r'\s+')
        if 'already_exists' == str(status) or 'registered' == str(status) or 'registered_without_ldap' == str(status):
            self.logger.debug('Current dn:' + dn)
#            msg = "Tahta başarı ile kaydedildi. \nLiderahenk Merkezi Yönetim Sistemine hoş geldiniz."
            self.update_messaging_conf(j)
            self.update_registration_attrs(dn)
            self.update_topic_name()
            
            # if "useSsh" in j:
            #     self.ssh_key.set_ssh_settings(j)
            # else:
            #     self.logger.info("useSSh is not exist in reply message")

            # Util.show_message(user, str(display_number), msg, "UYARI")
        elif 'registration_error' == str(status) or 'not_authorized' == str(status):
            self.logger.info('Registration is failed. New registration request will send')
            # msg = "Tahta Liderahenk Merkezi Yönetim Sistemine kaydedilemedi. Ağ ayarlarını kontrol edip tekrar kayıt işlemi için ETA yı yeniden başlatınız."
            self.logger.info("register_error_message is: " + str(self.register_error_message))
            # Util.show_message(user, str(display_number), msg, "UYARI")
            self.stop_service()
            self.re_register()
        else:
            self.logger.error('Bad message type of registration response')

    def update_registration_attrs(self, dn=None):
        # self.logger.debug('Registration configuration is updating...')
        self.db_service.update('registration', ['dn', 'registered'], [dn, 1], ' registered = 0')
        self.update_connection_conf()
        
                        
    def update_topic_name(self):
        self.logger.debug('Topic name is updating...')
        cols = ['topic_name', 'type', 'timestamp']
        vals = ['task-' + self.db_service.select_one_result('registration', 'jid', ' registered=1'), 
                'task', str(datetime.datetime.now().strftime("%d-%m-%Y %I:%M"))]

        self.db_service.delete('messaging', ' 1==1 ')
        self.db_service.update('messaging', cols, vals)
        

    def update_connection_conf(self):
        try:
            config = configparser.ConfigParser()
            config._interpolation = configparser.ExtendedInterpolation()
            config.read(System.Ahenk.config_path())

            config.set('CONNECTION', 'uid',
                                  self.db_service.select_one_result('registration', 'jid', ' registered=1'))
            config.set('CONNECTION', 'password', self.db_service.select_one_result('registration', 'password', ' registered=1'))
            with open(System.Ahenk.config_path(), 'w') as file:
                config.write(file)
            file.close()
            self.logger.debug('Registration configuration file is updated')
        except Exception as e:
            self.logger.error('Error while updating messaging conf file. Error Message {0}'.format(str(e)))
    
    
    def update_messaging_conf(self, response=None):
        self.logger.debug('Messaging conf file is updating...')
        try:
            config = configparser.ConfigParser()
            config._interpolation = configparser.ExtendedInterpolation()
            config.read(System.Ahenk.messaging_config_path())

            config.set('PULSAR', 'pulsar_host', str(response['messagingServerUrl']))
            config.set('PULSAR', 'pulsar_port', str(response['messagingServerPort']))
            config.set('PULSAR', 'pulsar_use_tls', str(response['messagingUseSsl']).lower())
            config.set('PULSAR', 'tls_trust_certs_file_path', str(response['clientSslPath']))
            config.set('MESSENGER', 'is_randomize_message', str(response['isRandomizeMessage']).lower())
            config.set('MESSENGER', 'randomize_time', str(response['messagingRandomizeTime']))

            with open(System.Ahenk.messaging_config_path(), 'w') as file:
                config.write(file)
            file.close()
        except Exception as e:
            self.logger.error('Error while updating messaging conf file. Error Message {0}'.format(str(e)))
        
    def is_registered(self):
        try:
            if str(System.Ahenk.uid()):
                return True
            else:
                return False
        except:
            return False

    def is_ldap_registered(self):
        dn = self.db_service.select_one_result('registration', 'dn', 'registered = 1')
        if dn is not None and dn != '':
            return True
        else:
            return False

    def register(self, uuid_depend_mac=False):

        cols = ['jid', 'password', 'registered', 'params', 'timestamp']
        vals = [str(self.generate_uuid(uuid_depend_mac)), str(self.generate_password()), 0,
                str(self.get_registration_params()), str(datetime.datetime.now().strftime("%d-%m-%Y %I:%M"))]

        self.db_service.delete('registration', ' 1==1 ')
        self.db_service.update('registration', cols, vals)
        self.logger.debug('Registration parameters were created')

    def get_registration_params(self):
        parts = []
        for part in System.Hardware.Disk.partitions():
            parts.append(part[0])

        params = {
            'ipAddresses': str(System.Hardware.Network.ip_addresses()).replace('[', '').replace(']', ''),
            'macAddresses': str(System.Hardware.Network.mac_addresses()).replace('[', '').replace(']', ''),
            'hostname': System.Os.hostname(),
            'os.name': System.Os.name(),
            'os.version': System.Os.version(),
            'os.kernel': System.Os.kernel_release(),
            'os.distributionName': System.Os.distribution_name(),
            'os.distributionId': System.Os.distribution_id(),
            'os.distributionVersion': System.Os.distribution_version(),
            'os.architecture': System.Os.architecture(),
            'hardware.cpu.architecture': System.Hardware.Cpu.architecture(),
            'hardware.cpu.logicalCoreCount': System.Hardware.Cpu.logical_core_count(),
            'hardware.cpu.physicalCoreCount': System.Hardware.Cpu.physical_core_count(),
            'hardware.disk.total': System.Hardware.Disk.total(),
            'hardware.disk.used': System.Hardware.Disk.used(),
            'hardware.disk.free': System.Hardware.Disk.free(),
            'hardware.disk.partitions': str(parts),
            'hardware.monitors': str(System.Hardware.monitors()),
            'hardware.screens': str(System.Hardware.screens()),
            'hardware.usbDevices': str(System.Hardware.usb_devices()),
            'hardware.printers': str(System.Hardware.printers()),
            'hardware.systemDefinitions': str(System.Hardware.system_definitions()),
            'hardware.memory.total': System.Hardware.Memory.total(),
            'hardware.network.ipAddresses': str(System.Hardware.Network.ip_addresses()),
            'sessions.userNames': str(System.Sessions.user_name()),
            'bios.releaseDate': System.BIOS.release_date()[1].replace('\n', '') if System.BIOS.release_date()[
                                                                                       0] == 0 else 'n/a',
            'bios.version': System.BIOS.version()[1].replace('\n', '') if System.BIOS.version()[0] == 0 else 'n/a',
            'bios.vendor': System.BIOS.vendor()[1].replace('\n', '') if System.BIOS.vendor()[0] == 0 else 'n/a',
            'hardware.baseboard.manufacturer': System.Hardware.BaseBoard.manufacturer()[1].replace('\n', '') if
            System.Hardware.BaseBoard.manufacturer()[0] == 0 else 'n/a',
            'hardware.baseboard.version': System.Hardware.BaseBoard.version()[1].replace('\n', '') if
            System.Hardware.BaseBoard.version()[0] == 0 else 'n/a',
            'hardware.baseboard.assetTag': System.Hardware.BaseBoard.asset_tag()[1].replace('\n', '') if
            System.Hardware.BaseBoard.asset_tag()[0] == 0 else 'n/a',
            'hardware.baseboard.productName': System.Hardware.BaseBoard.product_name()[1].replace('\n', '') if
            System.Hardware.BaseBoard.product_name()[0] == 0 else 'n/a',
            'hardware.baseboard.serialNumber': System.Hardware.BaseBoard.serial_number()[1].replace('\n', '') if
            System.Hardware.BaseBoard.serial_number()[0] == 0 else 'n/a',
            'phase': System.Hardware.Cpu.get_eta_phase(),
            'processor': System.Hardware.Cpu.brand(),
            'agentVersion': Util.get_agent_version(),
            'graphic.processing.unit': System.Hardware.Cpu.get_graphics_processing_unit(),
            # 'etaClassroomUnit' : System.Os.get_eta_classroom_unit(),
        }
        ssd_list, hdd_list = DiskInfo.get_all_disks()
        if len(ssd_list) > 0:
            params['hardware.disk.ssd.info'] = str(ssd_list)
        if len(hdd_list) > 0:
            params['hardware.disk.hdd.info'] = str(hdd_list)

        return json.dumps(params)

    def unregister(self):
        self.logger.debug('Ahenk is unregistering...')
        self.db_service.delete('registration', ' 1==1 ')
        self.logger.debug('Ahenk is unregistered')

    def clean(self):
        print('Ahenk cleaning..')
        import configparser
        try:
            config = configparser.ConfigParser()
            config._interpolation = configparser.ExtendedInterpolation()
            config.read(System.Ahenk.config_path())
            db_path = config.get('BASE', 'dbPath')

            if Util.is_exist(System.Ahenk.fifo_file()):
                Util.delete_file(System.Ahenk.fifo_file())

            if Util.is_exist(db_path):
                Util.delete_file(db_path)

            if Util.is_exist(System.Ahenk.pid_path()):
                Util.delete_file(System.Ahenk.pid_path())

            config.set('CONNECTION', 'uid', '')
            config.set('CONNECTION', 'password', '')
            config.set('CONNECTION', 'host', '')
            config.set('CONNECTION', 'servicename', 'false')
            config.set('MACHINE', 'user_disabled', 'false')

            with open(System.Ahenk.config_path(), 'w') as file:
                config.write(file)
            file.close()
            self.logger.error("Ahenk is cleaned")
        except Exception as e:
            self.logger.error("Error while running clean command. Error Message  " + str(e))
            print('Error while running clean command. Error Message {0}'.format(str(e)))

    def re_register(self):
        self.logger.debug('Reregistrating...')
        self.unregister()
        self.register(False)

    def generate_uuid(self, depend_mac=True):
        if depend_mac is False:
            self.logger.debug('uuid creating randomly')
            return uuid.uuid4()  # make a random UUID
        else:
            self.logger.debug('uuid creating according to mac address')
            return uuid.uuid3(uuid.NAMESPACE_DNS,
                              str(get_mac()))  # make a UUID using an MD5 hash of a namespace UUID and a mac address

    def generate_password(self):
        return uuid.uuid4()

    def registration_timeout(self):
        self.logger.error(
            'Could not reach registration response from Lider. Be sure XMPP server is reachable and it supports anonymous message, Lider is running properly '
            'and it is connected to XMPP server! Check your Ahenk configuration file (/etc/ahenk/ahenk.conf)')
        self.logger.error('Ahenk is shutting down...')
        print('Ahenk is shutting down...')
        System.Process.kill_by_pid(int(System.Ahenk.get_pid_number()))
